package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.DateUtils;
import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.EmsSite;
import com.xzzn.ems.domain.vo.SingleSiteBaseInfo;
import com.xzzn.ems.domain.vo.SiteMonitorDataVo;
import com.xzzn.ems.domain.vo.SiteTotalInfoVo;
import com.xzzn.ems.mapper.EmsPcsDataMapper;
import com.xzzn.ems.mapper.EmsSiteMapper;
import com.xzzn.ems.service.IEmsSiteService;
import com.xzzn.ems.service.IHomePageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 首页看板+站点地图 服务层实现
 *
 */
@Service
public class HomePageServiceImpl implements IHomePageService
{

    @Autowired
    private IEmsSiteService emsSiteService;
    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsSiteMapper emsSiteMapper;

    @Override
    public SiteTotalInfoVo getSiteTotalInfo() {
        return emsSiteService.getSiteTotalInfo();
    }

    @Override
    public SingleSiteBaseInfo getSingleSiteBaseInfo(Long siteId) {
        SingleSiteBaseInfo singleSiteBaseInfo = new SingleSiteBaseInfo();

        if (siteId != null) {
            // 站点基本信息
            EmsSite emsSite = emsSiteMapper.selectSiteById(siteId);
            if (emsSite != null) {
                singleSiteBaseInfo.setSiteLocation(emsSite.getRemark());//站点位置
                singleSiteBaseInfo.setRunningTime(DateUtils.parseDateToStr("yyyy-MM-dd",emsSite.getCreateTime()));//投运时间
                // 装机功率+装机容量 待定
                // 七天放电数据统计
                List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId);
                singleSiteBaseInfo.setSevenDayDischargeStats(siteMonitorDataVoList);
                // 充放电基本数据处理
                dealSitePCSDate(singleSiteBaseInfo,siteMonitorDataVoList);
            }
        }

        return singleSiteBaseInfo;
    }

    private void dealSitePCSDate(SingleSiteBaseInfo singleSiteBaseInfo, List<SiteMonitorDataVo> siteMonitorDataVoList) {
        if (siteMonitorDataVoList != null && !siteMonitorDataVoList.isEmpty()) {
            BigDecimal dayChargeCap = new BigDecimal(0);
            BigDecimal dayDisChargeCap = new BigDecimal(0);
            BigDecimal totalChargeCap = new BigDecimal(0);
            BigDecimal totalDisChargeCap = new BigDecimal(0);
            for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                // 总充电量
                totalChargeCap = totalChargeCap.add(sitePcsData.getChargedCap());
                // 总放电量
                totalDisChargeCap = totalDisChargeCap.add(sitePcsData.getDisChargedCap());
                // 获取当天的充电量+放电量
                String pcsDate = sitePcsData.getAmmeterDate();
                boolean isToday= checkIsToday(pcsDate);
                if(isToday){
                    dayChargeCap = dayChargeCap.add(sitePcsData.getChargedCap());
                    dayDisChargeCap = dayDisChargeCap.add(sitePcsData.getDisChargedCap());
                }
            }
            singleSiteBaseInfo.setDayChargedCap(dayChargeCap);
            singleSiteBaseInfo.setDayDisChargedCap(dayDisChargeCap);
            singleSiteBaseInfo.setTotalChargedCap(totalChargeCap);
            singleSiteBaseInfo.setTotalDischargedCap(totalDisChargeCap);
        }
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

}
