package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.EmsBatteryData;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.EmsAlarmRecordsMapper;
import com.xzzn.ems.mapper.EmsBatteryDataMapper;
import com.xzzn.ems.mapper.EmsPcsDataMapper;
import com.xzzn.ems.service.ISingleSiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.List;

/**
 * 单点监控 服务层实现
 */
@Service
public class SingleSiteServiceImpl implements ISingleSiteService {

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsAlarmRecordsMapper emsAlarmRecordsMapper;
    @Autowired
    private EmsBatteryDataMapper emsBatteryDataMapper;

    @Override
    public SiteMonitorHomeVo getSiteMonitorDataVo(Long siteId) {
        SiteMonitorHomeVo siteMonitorHomeVo = new SiteMonitorHomeVo();

        if (siteId != null) {
            // 实时告警数据 名称+状态+告警内容
            List<SiteMonitorHomeAlarmVo> siteMonitorHomeAlarmVo = emsAlarmRecordsMapper.getAlarmRecordsBySiteId(siteId);
            siteMonitorHomeVo.setSiteMonitorHomeAlarmVo(siteMonitorHomeAlarmVo);
            // 能量数据
            List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId);
            if (siteMonitorDataVoList != null && !siteMonitorDataVoList.isEmpty()) {
                BigDecimal dayChargeCap = new BigDecimal(0);
                BigDecimal dayDisChargeCap = new BigDecimal(0);
                BigDecimal totalChargeCap = new BigDecimal(0);
                BigDecimal totalDisChargeCap = new BigDecimal(0);
                for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                    // 总充电量
                    totalChargeCap = totalChargeCap.add(sitePcsData.getChargedCap());
                    // 总放电量
                    totalDisChargeCap = totalDisChargeCap.add(sitePcsData.getDisChargedCap());
                    // 获取当天的充电量+放电量
                    String pcsDate = sitePcsData.getAmmeterDate();
                    boolean isToday= checkIsToday(pcsDate);
                    if(isToday){
                        dayChargeCap = dayChargeCap.add(sitePcsData.getChargedCap());
                        dayDisChargeCap = dayDisChargeCap.add(sitePcsData.getDisChargedCap());
                    }
                }
                siteMonitorHomeVo.setDayChargedCap(dayChargeCap);
                siteMonitorHomeVo.setDayDisChargedCap(dayDisChargeCap);
                siteMonitorHomeVo.setTotalChargedCap(totalChargeCap);
                siteMonitorHomeVo.setTotalDischargedCap(totalDisChargeCap);
                // 储能可用电量
                BigDecimal energyStorageAvailElec = siteMonitorHomeVo.getTotalDischargedCap().subtract(siteMonitorHomeVo.getTotalChargedCap());
                siteMonitorHomeVo.setEnergyStorageAvailElec(energyStorageAvailElec);
            }
            siteMonitorHomeVo.setSiteMonitorDataVo(siteMonitorDataVoList);
            // 电网实时功率
            siteMonitorHomeVo.setGridNrtPower(emsPcsDataMapper.getGridNrtPower(siteId));
        }

        return siteMonitorHomeVo;
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    @Override
    public SiteMonitorRuningHeadInfoVo getSiteRuningHeadInfo(Long siteId) {
        SiteMonitorRuningHeadInfoVo siteMonitorRuningHeadInfoVo = new SiteMonitorRuningHeadInfoVo();

        if (siteId != null) {
            // 实时有功功率/实时无功功率/今日充电量/今日放电量
            siteMonitorRuningHeadInfoVo = emsPcsDataMapper.getSiteRunningHeadInfo(siteId);
            // 电池簇SOC
            // 电池堆SOH
            EmsBatteryData emsBatteryData = emsBatteryDataMapper.getBatteryDataBySiteId(siteId);
            if (emsBatteryData != null) {
                siteMonitorRuningHeadInfoVo.setSoc(emsBatteryData.getSoc());
                siteMonitorRuningHeadInfoVo.setSoh(emsBatteryData.getSoh());
            }
        }

        return siteMonitorRuningHeadInfoVo;
    }

}
